==> tkit-quarkus-rhpam-client-2.9.0-sources.jar: org/tkit/rhpam/quarkus/rs/ProcessInstanceRestController.java <==
package org.tkit.rhpam.quarkus.rs;

import lombok.extern.slf4j.Slf4j;
import ext.api.centrallog.api.ProcessLogEventEmitter;
import org.eclipse.microprofile.config.inject.ConfigProperty;
import org.eclipse.microprofile.openapi.annotations.tags.Tag;
import org.eclipse.microprofile.rest.client.inject.RestClient;
import org.tkit.rhpam.quarkus.domain.daos.DomainProcessInfoDAO;
import org.tkit.rhpam.quarkus.domain.models.DomainProcessInfo;
import org.tkit.rhpam.quarkus.messaging.common.RhpamException;
import org.tkit.rhpam.quarkus.rs.clients.KieServerRestClient;
import org.tkit.rhpam.quarkus.rs.model.ProcessInstanceDTO;
import org.tkit.rhpam.quarkus.rs.model.ProcessInstanceRemovalRequest;
import org.tkit.rhpam.quarkus.rs.model.ProcessInstanceWrapper;

import javax.enterprise.context.RequestScoped;
import javax.inject.Inject;
import javax.transaction.Transactional;
import javax.ws.rs.*;
import javax.ws.rs.core.GenericType;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.stream.Collectors;

@RequestScoped
@Transactional
@Path("/processInstances")
@Tag(name = "processInstanceAPI")
@Produces(MediaType.APPLICATION_JSON)
@Consumes(MediaType.APPLICATION_JSON)
@Slf4j
public class ProcessInstanceRestController {

    @Inject
    private ProcessLogEventEmitter processLogEventEmitter;

    @Inject
    private DomainProcessInfoDAO domainProcessInfoDAO;

    @Inject
    @RestClient
    private KieServerRestClient kieServerRestClient;

    @ConfigProperty(name = "kie.admin.user")
    private Optional<String> usernameParam;

    @ConfigProperty(name = "kie.admin.pwd")
    private Optional<String> passwordParam;

    @ConfigProperty(name = "org.tkit.rhpam.abortSubprocessRecursionLevel")
    private Optional<Integer> recursiveLevelParam;

    private static final Integer MAX_DEFAULT_RECURSIVE_LEVEL = 10;


    @POST
    @Path("/{processInstanceId}/abortRequest")
    public Response abortProcess(@PathParam("processInstanceId") Long processInstanceId, ProcessInstanceRemovalRequest removalRequest) throws RhpamException {
        String username = usernameParam.map(Object::toString).orElseThrow(() -> new RhpamException("Kie server username can not be null. " +
                "Please provide property kie.admin.user with correct value", Response.Status.BAD_REQUEST));
        String password = passwordParam.map(Object::toString).orElseThrow(() -> new RhpamException("Kie server password can not be null." +
                "Please provide property kie.admin.pwd with correct value", Response.Status.BAD_REQUEST));
        Integer recursiveLevelLimit = recursiveLevelParam.orElse(MAX_DEFAULT_RECURSIVE_LEVEL);
        String authentication = getClientCredentialsAuth(username, password);
        log.debug("Aborting by process instance {} with message {}", processInstanceId, removalRequest.getMessage());
        cleanDomainProcessInfo(processInstanceId, removalRequest.isDeleteDomainProcessInfo());
        //Do not try to change the order of execution of abortMainInstance and abortChildInstances
        //Parent process must be aborted first
        abortMainInstance(authentication, processInstanceId, removalRequest);
        abortChildInstances(authentication, processInstanceId, removalRequest.getContainerId(), 0, recursiveLevelLimit);
        return Response.accepted().build();
    }

    private void abortMainInstance(String authentication, Long processInstanceId, ProcessInstanceRemovalRequest removalRequest) throws RhpamException {
        ProcessInstanceDTO processInstance;
        try {
            processInstance = kieServerRestClient.getProcessInstance(authentication, removalRequest.getContainerId(), processInstanceId)
                    .readEntity(ProcessInstanceDTO.class);
        } catch (Exception exception) {
            throw new RhpamException("Process instance with id " + processInstanceId + " does not exist.", Response.Status.NOT_FOUND);
        }
        if (processInstance.getState() == 1) {
            try( Response response = kieServerRestClient.abortProcessInstance(authentication, processInstance.getContainerId(), processInstanceId)) {
                if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
                    log.info("Process instance with id {} succesfully aborted", processInstanceId);
                } else {
                    log.warn("Process instance with id {} failed to be aborted, responseCode {}", processInstanceId, response.getStatus());
                }
            }
        }
    }

    private void abortChildInstances(String authentication, Long processInstanceId, String containerId, int currentRecursiveLevel, Integer recursiveLevelLimit) throws RhpamException {
        List<ProcessInstanceDTO> childInstances = kieServerRestClient.getProcessInstances(authentication, containerId, processInstanceId)
                .readEntity(new GenericType<ProcessInstanceWrapper>() {
                }).getProcessInstances()
                .stream()
                .filter(instance -> instance.getState() == 1)
                .collect(Collectors.toList());
        List<Long> processInstanceIdsToAbort = childInstances
                .stream()
                .map(ProcessInstanceDTO::getId)
                .collect(Collectors.toList());
        if (!processInstanceIdsToAbort.isEmpty() && currentRecursiveLevel < recursiveLevelLimit) {
            log.debug("Aborting {} subprocesses with following instance ids {}", processInstanceIdsToAbort.size(), processInstanceIdsToAbort);
            try (Response response = kieServerRestClient.abortProcessInstances(authentication, containerId, processInstanceIdsToAbort)) {
                if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
                    log.info("Process instance with id {} succesfully aborted", processInstanceIdsToAbort);
                } else {
                    log.warn("Process instance with id {} failed to be aborted, responseCode {}", processInstanceId, response.getStatus());
                }
            }
            currentRecursiveLevel++;
            for (Long processInstanceIdToAbort:processInstanceIdsToAbort) {
                abortChildInstances(authentication, processInstanceIdToAbort, containerId, currentRecursiveLevel, recursiveLevelLimit);
            }
        }
    }

    private void cleanDomainProcessInfo(Long processInstanceId, boolean deleteDomainProcessInfo) {
        DomainProcessInfo domainProcessInfo = domainProcessInfoDAO.selectForUpdate(processInstanceId);
        if (domainProcessInfo != null) {
            if (deleteDomainProcessInfo) {
                domainProcessInfoDAO.delete(domainProcessInfo);
            } else {
                domainProcessInfo.setCurrentProcessStatus(DomainProcessInfo.ProcessStatus.ABORTED);
                domainProcessInfo.getStageFlags().forEach((s, stageFlag) -> {
                    if (DomainProcessInfo.ProcessStatus.RUNNING.equals(stageFlag.getStatus())) {
                        stageFlag.setStatus(DomainProcessInfo.ProcessStatus.ABORTED);
                    }
                });
                domainProcessInfoDAO.update(domainProcessInfo);
            }
        } else {
            log.warn("Domain process info with process instance id {} not found.", processInstanceId);
        }
    }

    private String getClientCredentialsAuth(String username, String password) {
        String tmp = username + ":" + password;
        return "Basic " + Base64.getEncoder().encodeToString(tmp.getBytes(StandardCharsets.UTF_8));
    }

}

==> tkit-quarkus-rhpam-client-2.12.0-sources.jar: org/tkit/rhpam/quarkus/rs/ProcessInstanceRestController.java <==
package org.tkit.rhpam.quarkus.rs;

import lombok.extern.slf4j.Slf4j;
import ext.api.centrallog.api.ProcessLogEventEmitter;
import org.eclipse.microprofile.config.inject.ConfigProperty;
import org.eclipse.microprofile.openapi.annotations.tags.Tag;
import org.eclipse.microprofile.rest.client.inject.RestClient;
import org.tkit.rhpam.quarkus.domain.daos.DomainProcessInfoDAO;
import org.tkit.rhpam.quarkus.domain.models.DomainProcessInfo;
import org.tkit.rhpam.quarkus.messaging.common.RhpamException;
import org.tkit.rhpam.quarkus.rs.clients.KieServerRestClient;
import org.tkit.rhpam.quarkus.rs.model.ProcessInstanceDTO;
import org.tkit.rhpam.quarkus.rs.model.ProcessInstanceRemovalRequest;
import org.tkit.rhpam.quarkus.rs.model.ProcessInstanceWrapper;

import javax.enterprise.context.RequestScoped;
import javax.inject.Inject;
import javax.transaction.Transactional;
import javax.ws.rs.*;
import javax.ws.rs.core.GenericType;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.stream.Collectors;

@RequestScoped
@Transactional
@Path("/processInstances")
@Tag(name = "processInstanceAPI")
@Produces(MediaType.APPLICATION_JSON)
@Consumes(MediaType.APPLICATION_JSON)
@Slf4j
public class ProcessInstanceRestController {

    @Inject
    private ProcessLogEventEmitter processLogEventEmitter;

    @Inject
    private DomainProcessInfoDAO domainProcessInfoDAO;

    @Inject
    @RestClient
    private KieServerRestClient kieServerRestClient;

    @ConfigProperty(name = "kie.admin.user")
    private Optional<String> usernameParam;

    @ConfigProperty(name = "kie.admin.pwd")
    private Optional<String> passwordParam;

    @ConfigProperty(name = "org.tkit.rhpam.abortSubprocessRecursionLevel")
    private Optional<Integer> recursiveLevelParam;

    private static final Integer MAX_DEFAULT_RECURSIVE_LEVEL = 10;


    @POST
    @Path("/{processInstanceId}/abortRequest")
    public Response abortProcess(@PathParam("processInstanceId") Long processInstanceId, ProcessInstanceRemovalRequest removalRequest) throws RhpamException {
        String username = usernameParam.map(Object::toString).orElseThrow(() -> new RhpamException("Kie server username can not be null. " +
                "Please provide property kie.admin.user with correct value", Response.Status.BAD_REQUEST));
        String password = passwordParam.map(Object::toString).orElseThrow(() -> new RhpamException("Kie server password can not be null." +
                "Please provide property kie.admin.pwd with correct value", Response.Status.BAD_REQUEST));
        Integer recursiveLevelLimit = recursiveLevelParam.orElse(MAX_DEFAULT_RECURSIVE_LEVEL);
        String authentication = getClientCredentialsAuth(username, password);
        log.debug("Aborting by process instance {} with message {}", processInstanceId, removalRequest.getMessage());
        cleanDomainProcessInfo(processInstanceId, removalRequest.isDeleteDomainProcessInfo());
        //Do not try to change the order of execution of abortMainInstance and abortChildInstances
        //Parent process must be aborted first
        abortMainInstance(authentication, processInstanceId, removalRequest);
        abortChildInstances(authentication, processInstanceId, removalRequest.getContainerId(), 0, recursiveLevelLimit);
        return Response.accepted().build();
    }

    private void abortMainInstance(String authentication, Long processInstanceId, ProcessInstanceRemovalRequest removalRequest) throws RhpamException {
        ProcessInstanceDTO processInstance;
        try {
            processInstance = kieServerRestClient.getProcessInstance(authentication, removalRequest.getContainerId(), processInstanceId)
                    .readEntity(ProcessInstanceDTO.class);
        } catch (Exception exception) {
            throw new RhpamException("Process instance with id " + processInstanceId + " does not exist.", Response.Status.NOT_FOUND);
        }
        if (processInstance.getState() == 1) {
            try( Response response = kieServerRestClient.abortProcessInstance(authentication, processInstance.getContainerId(), processInstanceId)) {
                if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
                    log.info("Process instance with id {} successfully aborted", processInstanceId);
                } else {
                    log.warn("Process instance with id {} failed to be aborted, responseCode {}", processInstanceId, response.getStatus());
                }
            }
        }
    }

    private void abortChildInstances(String authentication, Long processInstanceId, String containerId, int currentRecursiveLevel, Integer recursiveLevelLimit) throws RhpamException {
        List<ProcessInstanceDTO> childInstances = kieServerRestClient.getProcessInstances(authentication, containerId, processInstanceId)
                .readEntity(new GenericType<ProcessInstanceWrapper>() {
                }).getProcessInstances()
                .stream()
                .filter(instance -> instance.getState() == 1)
                .collect(Collectors.toList());
        List<Long> processInstanceIdsToAbort = childInstances
                .stream()
                .map(ProcessInstanceDTO::getId)
                .collect(Collectors.toList());
        if (!processInstanceIdsToAbort.isEmpty() && currentRecursiveLevel < recursiveLevelLimit) {
            try (Response response = kieServerRestClient.abortProcessInstances(authentication, containerId, processInstanceIdsToAbort)) {
                if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
                    log.info("Process instance with ids {} successfully aborted", processInstanceIdsToAbort);
                } else {
                    log.warn("Process instance with ids {} failed to be aborted, responseCode {}", processInstanceIdsToAbort, response.getStatus());
                }
            }
            currentRecursiveLevel++;
            for (Long processInstanceIdToAbort:processInstanceIdsToAbort) {
                abortChildInstances(authentication, processInstanceIdToAbort, containerId, currentRecursiveLevel, recursiveLevelLimit);
            }
        }
    }

    private void cleanDomainProcessInfo(Long processInstanceId, boolean deleteDomainProcessInfo) {
        DomainProcessInfo domainProcessInfo = domainProcessInfoDAO.selectForUpdate(processInstanceId);
        if (domainProcessInfo != null) {
            if (deleteDomainProcessInfo) {
                domainProcessInfoDAO.delete(domainProcessInfo);
            } else {
                domainProcessInfo.setCurrentProcessStatus(DomainProcessInfo.ProcessStatus.ABORTED);
                domainProcessInfo.getStageFlags().forEach((s, stageFlag) -> {
                    if (DomainProcessInfo.ProcessStatus.RUNNING.equals(stageFlag.getStatus())) {
                        stageFlag.setStatus(DomainProcessInfo.ProcessStatus.ABORTED);
                    }
                });
                domainProcessInfoDAO.update(domainProcessInfo);
            }
        } else {
            log.warn("Domain process info with process instance id {} not found.", processInstanceId);
        }
    }

    private String getClientCredentialsAuth(String username, String password) {
        String tmp = username + ":" + password;
        return "Basic " + Base64.getEncoder().encodeToString(tmp.getBytes(StandardCharsets.UTF_8));
    }

}
